"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScanService = void 0;
var context_map_server_client_1 = require("../infrastructure/client/context-map-server.client");
var scan_1 = require("../model/scan");
var resource_service_1 = require("./resource.service");
var technology_stack_service_1 = require("./technology-stack.service");
var actor_service_1 = require("./actor.service");
var ts_morph_1 = require("ts-morph");
var subscribed_endpoint_service_1 = require("./subscribed-endpoint.service");
var languages_service_1 = require("./languages.service");
var glossary_service_1 = require("./glossary.service");
var entity_service_1 = require("./entity.service");
var version_control_service_1 = require("./version-control.service");
var decision_record_service_1 = require("./decision-record.service");
var feature_file_service_1 = require("./feature-file.service");
var exemplar_service_1 = require("./exemplar.service");
var capability_service_1 = require("./capability.service");
var contextmapPackageJson = require('../../package.json');
var ScanService = /** @class */ (function () {
    function ScanService(parameters) {
        this.parameters = parameters;
        this.tagCache = [];
        this.decisionRecordFilenameForOverview = '';
        this.decisionRecordsCache = [];
        this.featureFilesCache = [];
        this.project = new ts_morph_1.Project();
        this.resourceService = new resource_service_1.ResourceService();
        this.configurationResource = this.resourceService.scanConfiguration();
        if (parameters.customHost == null) {
            parameters.customHost = this.configurationResource.getHost();
        }
        this.client = new context_map_server_client_1.ContextMapServerClient(parameters);
        this.technologyStackService = new technology_stack_service_1.TechnologyStackService();
        // this.decoratorService = new DecoratorService(this.project)
        this.actorService = new actor_service_1.ActorService(this.project);
        this.capabilityService = new capability_service_1.CapabilityService(this.project);
        this.glossaryService = new glossary_service_1.GlossaryService(this.project);
        this.exemplarService = new exemplar_service_1.ExemplarService(this.project);
        this.entityService = new entity_service_1.EntityService(this.project);
        this.versionControlService = new version_control_service_1.VersionControlService();
        this.decisionRecordService = new decision_record_service_1.DecisionRecordService(this.project);
        this.featureFileService = new feature_file_service_1.FeatureFileService(this.project);
        this.subscribedEndpointService = new subscribed_endpoint_service_1.SubscribedEndpointService(this.project);
        this.languageService = new languages_service_1.LanguagesService(this.project);
    }
    ScanService.prototype.scan = function () {
        var scan = this.initializeScan();
        this.scanConfiguration(scan);
        this.scanTechStack(scan);
        this.scanActors(scan);
        this.scanCapabilities(scan);
        this.scanSubscribedEndpoints(scan);
        this.scanBytesOfCode(scan);
        this.scanGlossaryTerms(scan);
        this.scanExemplars(scan);
        this.scanEntities(scan);
        this.scanVersionControl(scan);
        this.scanDecisionRecords(scan);
        this.scanFeatureFiles(scan);
        // this.scanDecorators(scan)
        var optResponse = this.sendScan(scan);
        if (optResponse != null) {
            this.sendDecisionRecordDetails(optResponse.componentId, optResponse.parentComponentId, scan.decisionRecordOverviews, optResponse.decisionRecordsToSend);
            this.sendFeatureFileDetails(optResponse.componentId, optResponse.parentComponentId, optResponse.featureFilesToSend);
            this.sendReleases(optResponse.componentId, optResponse.parentComponentId, optResponse.releasesToSend);
        }
    };
    ScanService.prototype.initializeScan = function () {
        console.log('Scanning sources at: ' + this.parameters.source);
        try {
            if (typeof this.parameters.source === 'string') {
                this.project.addSourceFilesAtPaths(this.parameters.source);
            }
            else {
                var sources = this.parameters.source;
                this.project.addSourceFilesAtPaths(sources);
            }
        }
        catch (error) {
            console.error('Error while initializing project sourcefiles');
        }
        return (0, scan_1.createScan)(contextmapPackageJson.version);
    };
    ScanService.prototype.scanConfiguration = function (scan) {
        scan.execution.scannedProperties = true;
        scan.execution.scannedSystem = true;
        if (this.parameters.componentName != null && this.parameters.componentName.length > 0) {
            scan.scannedComponent.name = this.parameters.componentName;
        }
        else {
            scan.scannedComponent.name = this.configurationResource.getName();
        }
        if (this.parameters.systemName != null && this.parameters.systemName.length > 0) {
            scan.scannedComponent.systemName = this.parameters.systemName;
        }
        else {
            scan.scannedComponent.systemName = this.configurationResource.getSystemName();
        }
        scan.scannedComponent.version = this.configurationResource.getVersion();
        scan.scannedComponent.team = this.configurationResource.getTeam();
        scan.scannedComponent.organization = this.configurationResource.getOrganization();
        scan.scannedComponent.teamMail = this.configurationResource.getTeamMail();
        scan.scannedComponent.domainVisionStatement = this.configurationResource.getDomainVisionStatement();
        scan.scannedComponent.type = this.configurationResource.getComponentType();
        scan.scannedComponent.aliases = this.configurationResource.getAliases();
        scan.scannedComponent.urlBuildPipeline = this.configurationResource.getUrlBuildPipeline();
        scan.scannedComponent.urlDocumentation = this.configurationResource.getUrlDocumentation();
        scan.scannedComponent.urlIssueManagement = this.configurationResource.getUrlIssueManagement();
        scan.scannedComponent.urlVersionControl = this.configurationResource.getUrlVersionControl();
        this.decisionRecordFilenameForOverview = this.configurationResource.getDecisionRecordFilenameForOverview();
        // Scanners which depend on the config resource
        this.scanTechRadar(scan, this.configurationResource);
    };
    // private scanDecorators(scan: Scan): void {
    //   console.info("Scanning entities");
    //   scan.execution.scannedEntities = true;
    //   scan.entities = this.decoratorService.scan();
    //   console.info(" < Found " + scan.entities.length + " entities");
    // }
    ScanService.prototype.scanTechRadar = function (scan, configurationResource) {
        console.info("Scanning for tech-radar");
        scan.execution.scannedTechRadarEntries = true;
        scan.techRadarEntries = configurationResource.getTechRadarEntries();
        console.info(" < Found " + scan.techRadarEntries.length + " tech-radar entries");
    };
    ScanService.prototype.scanBytesOfCode = function (scan) {
        console.info("Scanning bytes of code");
        scan.execution.scannedBytesOfCode = true;
        scan.scannedComponent.bytesOfCodes = this.languageService.getBytesOfCodePerLanguage();
        console.info(" < Found " + scan.scannedComponent.bytesOfCodes.length + " languages");
    };
    ScanService.prototype.scanGlossaryTerms = function (scan) {
        console.info("Scanning glossary terms");
        scan.execution.scannedGlossary = true;
        scan.glossaryTerms = this.glossaryService.scan();
        console.info(" < Found " + scan.glossaryTerms.length + " glossary terms");
    };
    ScanService.prototype.scanExemplars = function (scan) {
        console.info("Scanning exemplars");
        scan.execution.scannedExemplars = true;
        scan.exemplars = this.exemplarService.scan();
        console.info(" < Found " + scan.exemplars.length + " exemplars");
    };
    ScanService.prototype.scanDecisionRecords = function (scan) {
        console.info("Scanning for architecture decision records");
        scan.execution.scannedDecisionRecords = true;
        this.decisionRecordsCache = this.decisionRecordService.scan();
        scan.decisionRecordOverviews = this.decisionRecordsCache.map(function (o) { return o.decisionRecordOverview; });
        console.info(" < Found " + scan.decisionRecordOverviews.length + " decision records");
    };
    ScanService.prototype.scanFeatureFiles = function (scan) {
        console.info("Scanning for feature files");
        scan.execution.scannedFeatureFiles = true;
        this.featureFilesCache = this.featureFileService.scan();
        scan.featureFileOverviews = this.featureFilesCache.map(function (o) { return o.featureFileOverview; });
        console.info(" < Found " + scan.featureFileOverviews.length + " feature files");
    };
    ScanService.prototype.scanVersionControl = function (scan) {
        console.info("Scanning version control");
        var versionControlScan = this.versionControlService.scan(scan.scannedComponent.name, this.client);
        scan.execution.scannedCommitLog = true;
        scan.commitDetails = versionControlScan.commitDetailList;
        scan.scannedComponent.recentCommits = versionControlScan.commitOverviewList;
        console.info(" < Found " + scan.commitDetails.length + " recent commits");
        scan.execution.scannedReleases = true;
        this.tagCache = versionControlScan.tagList;
        scan.releaseOverviews = this.tagCache.map(function (tag) {
            var ro = {
                name: tag.name,
                createdOn: (0, version_control_service_1.formatTagCreatedOn)(tag)
            };
            return ro;
        });
        console.info(" < Found " + scan.releaseOverviews.length + " unscanned releases");
    };
    ScanService.prototype.scanEntities = function (scan) {
        console.info("Scanning entities");
        scan.execution.scannedEntities = true;
        scan.entities = this.entityService.scan();
        console.info(" < Found " + scan.entities.length + " entities");
    };
    ScanService.prototype.scanSubscribedEndpoints = function (scan) {
        console.info("Scanning subscribed endpoints");
        scan.execution.scannedSubscribedEndpoints = true;
        scan.subscribedEndpoints = this.subscribedEndpointService.scan();
        console.info(" < Found " + scan.subscribedEndpoints.length + " subscribed endpoints");
    };
    ScanService.prototype.scanActors = function (scan) {
        console.info("Scanning actors");
        scan.execution.scannedActors = true;
        scan.actors = this.actorService.scan();
        console.info(" < Found " + scan.actors.length + " actors");
    };
    ScanService.prototype.scanCapabilities = function (scan) {
        console.info("Scanning capabilities");
        scan.execution.scannedCapabilities = true;
        scan.capabilities = this.capabilityService.scan(this.configurationResource);
        console.info(" < Found " + scan.capabilities.length + " capabilities");
    };
    ScanService.prototype.scanTechStack = function (scan) {
        console.info("Scanning techstack");
        scan.execution.scannedTechnologies = true;
        scan.scannedComponent.technologies = this.technologyStackService.scan();
        console.info(" < Found " + scan.scannedComponent.technologies.length + " technologies");
    };
    ScanService.prototype.sendScan = function (scan) {
        return this.client.processScan(scan);
    };
    ScanService.prototype.sendFeatureFileDetails = function (componentId, parentComponentId, requestedOverviews) {
        var _this = this;
        if (requestedOverviews == null) {
            console.info("Server requested no feature file details");
            return;
        }
        requestedOverviews.map(function (requestedFF) {
            return _this.featureFilesCache.find(function (ff) { return ff.featureFileOverview.contentHash === requestedFF.contentHash; });
        })
            .filter(function (scannedFF) { return scannedFF != null; })
            .map(function (scannedFF) { return _this.featureFileService.createFeatureFileDetail(componentId, parentComponentId, scannedFF); })
            .forEach(function (ffDetail) {
            console.info("Sending details of feature file " + ffDetail.name + " requested by server");
            _this.client.addFeatureFile(ffDetail);
        });
    };
    ScanService.prototype.setDecisionRecordForOverview = function (componentId, parentComponentId, scannedOverviews) {
        var _this = this;
        // TODO
        try {
            if (this.decisionRecordFilenameForOverview == null || this.decisionRecordFilenameForOverview.length === 0) {
                console.info("Clearing decision record for overview");
                var adrDetail = {
                    contentHash: null,
                    componentId: componentId,
                    parentComponentId: parentComponentId
                };
                this.client.setDecisionRecordForOverview(adrDetail);
            }
            else {
                var hashes = scannedOverviews.filter(function (scannedOverview) { return scannedOverview.name === _this.decisionRecordFilenameForOverview; })
                    .map(function (scannedOverview) { return scannedOverview.contentHash; });
                var contentHash = (hashes == null || hashes.length === 0) ? null : hashes[0];
                console.info("Setting decision record for overview (hash: " + contentHash + ")");
                var adrDetail = {
                    contentHash: contentHash,
                    componentId: componentId,
                    parentComponentId: parentComponentId
                };
                this.client.setDecisionRecordForOverview(adrDetail);
            }
        }
        catch (e) {
            console.error("Unable to set decision record for overview: " + e);
        }
    };
    ScanService.prototype.sendDecisionRecordDetails = function (componentId, parentComponentId, scannedOverviews, requestedOverviews) {
        var _this = this;
        if (requestedOverviews == null) {
            console.info("Server requested no decision records details");
            this.setDecisionRecordForOverview(componentId, parentComponentId, scannedOverviews);
            return;
        }
        requestedOverviews.map(function (requestedADR) {
            return _this.decisionRecordsCache.find(function (dr) { return dr.decisionRecordOverview.contentHash === requestedADR.contentHash; });
        })
            .filter(function (scannedADR) { return scannedADR != null; })
            .map(function (scannedADR) { return _this.decisionRecordService.createDecisionRecordDetail(componentId, parentComponentId, scannedADR); })
            .forEach(function (adrDetail) {
            console.info("Sending details of decision record " + adrDetail.name + " requested by server");
            _this.client.addDecisionRecord(adrDetail);
        });
        this.setDecisionRecordForOverview(componentId, parentComponentId, scannedOverviews);
    };
    ScanService.prototype.sendReleases = function (componentId, parentComponentId, requestedReleases) {
        var _this = this;
        var requestedTagNames = new Set();
        requestedReleases
            .map(function (r) { return r.name; })
            .forEach(function (n) { return requestedTagNames.add(n); });
        this.tagCache
            .filter(function (tag) { return requestedTagNames.has(tag.name); })
            .forEach(function (tag) {
            var releaseDetail = {
                componentId: componentId,
                parentComponentId: parentComponentId,
                name: tag.name,
                createdOn: (0, version_control_service_1.formatTagCreatedOn)(tag),
                commitDetails: tag.commits,
            };
            console.log('Sending details of release ' + tag.name + ' requested by server');
            _this.client.addRelease(releaseDetail);
        });
    };
    return ScanService;
}());
exports.ScanService = ScanService;
